@@ -461,7 +461,9 @@ seed_gobject_method_invoked (JSContextRef ctx,
   GArgument *in_args;
   GArgument *out_args;
   GArgument *out_values;
-  guint n_args, n_in_args, n_out_args, i;
+  gint first_out = -1;
+  guint use_return_as_out = 0;
+  guint n_args, n_in_args, n_out_args, i; 
   guint in_args_pos, out_args_pos;
   GIArgInfo *arg_info;
   GITypeInfo *type_info;
@@ -489,20 +491,79 @@ seed_gobject_method_invoked (JSContextRef ctx,
 
   for (i = 0; (i < (n_args)); i++)
     {
-      SEED_NOTE (INVOCATION,
-		 "Converting arg: %d of function %s, exception is %p", i,
-		 g_base_info_get_name (info), exception);
+
       arg_info = g_callable_info_get_arg ((GICallableInfo *) info, i);
       dir = g_arg_info_get_direction (arg_info);
       type_info = g_arg_info_get_type (arg_info);
+
+      SEED_NOTE (INVOCATION,
+                "Converting arg: %s (%d) of function %s, exception is %p",  
+                 g_base_info_get_name ((GIBaseInfo *)arg_info),  i,
+                 g_base_info_get_name (info), exception);
+
+
+
       if (i + 1 > argumentCount)
-	{
-	  in_args[n_in_args++].v_pointer = 0;
+        {
+
+          if (dir == GI_DIRECTION_OUT)
+            {
+              GArgument *out_value = &out_values[n_out_args];
+              out_args[n_out_args++].v_pointer = out_value;
+            } 
+          else
+              in_args[n_in_args++].v_pointer = 0;   
+
 	}
       else if (dir == GI_DIRECTION_IN || dir == GI_DIRECTION_INOUT)
 	{
 
-	  if (!seed_gi_make_argument (ctx, arguments[i], type_info,
+	  if (  !g_arg_info_may_be_null (arg_info) ) 
+            {
+  	      gboolean is_null = ( !arguments[i] || JSValueIsNull (ctx, arguments[i]) );
+
+              if (!is_null && (g_type_info_get_tag (type_info) == GI_TYPE_TAG_INTERFACE)) 
+                {
+                  /* see if the pointer is null for struct/unions. */
+                  GIBaseInfo *interface = g_type_info_get_interface (type_info);
+                  GIInfoType interface_type = g_base_info_get_type (interface);
+                  
+		  gboolean arg_is_object = JSValueIsObject (ctx, arguments[i]);
+		  gboolean is_struct_or_union = (
+		           interface_type == GI_INFO_TYPE_STRUCT ||
+		           interface_type == GI_INFO_TYPE_UNION
+		  );
+
+	          /* this test ignores non-objects being sent where interfaces are expected 
+	             hopefully our type manipluation code will pick that up. */
+                  if (is_struct_or_union && arg_is_object &&
+                        (seed_pointer_get_pointer (ctx, arguments[i]) == 0)) 
+                      is_null = TRUE;
+                  
+                  g_base_info_unref (interface);
+                }
+
+              if (is_null) 
+                {
+                  GIBaseInfo *ctr = g_base_info_get_container((GIBaseInfo *) info);
+                  // note - ctr does not need unref'ing (see ginfo.c source for why)
+
+                  // RE-INSTATE THIS CODE LATER.. - when gtk etc. has be release with fixes
+		  //seed_make_exception (ctx, exception,
+		  //   "ArgumentError",
+                  g_warning(      "ArgumentError - probably due to incorrect gir file (which may be fixed upstream)"
+				   " argument %d must not be null for"
+				   " function: %s%s%s \n",
+				   i + 1,
+				   ctr ? g_base_info_get_name ((GIBaseInfo *) ctr) : "",
+				   ctr ? "." : "",
+				   g_base_info_get_name ((GIBaseInfo *)
+							 info));
+                  //goto arg_error;
+                }
+            }
+
+          if (!seed_gi_make_argument (ctx, arguments[i], type_info,
 				      &in_args[n_in_args++], exception))
 	    {
 	      seed_make_exception (ctx, exception,
@@ -513,6 +574,8 @@ seed_gobject_method_invoked (JSContextRef ctx,
 				   g_base_info_get_name ((GIBaseInfo *)
 							 info));
 
+// FIXME - SEE NOTE ABOVE ABOUT gtk allow_null bugs
+// arg_error:
 	      g_base_info_unref ((GIBaseInfo *) type_info);
 	      g_base_info_unref ((GIBaseInfo *) arg_info);
 	      g_free (in_args);
@@ -532,13 +595,15 @@ seed_gobject_method_invoked (JSContextRef ctx,
 	{
 	  GArgument *out_value = &out_values[n_out_args];
 	  out_args[n_out_args++].v_pointer = out_value;
+	  first_out = first_out > -1 ? first_out : i;
+
 	}
 
       g_base_info_unref ((GIBaseInfo *) type_info);
       g_base_info_unref ((GIBaseInfo *) arg_info);
     }
-  SEED_NOTE (INVOCATION, "Invoking method: %s with %d in arguments"
-	     " and %d out arguments",
+  SEED_NOTE (INVOCATION, "Invoking method: %s with %d 'in' arguments"
+	     " and %d 'out' arguments",
 	     g_base_info_get_name (info), n_in_args, n_out_args);
   if (g_function_info_invoke ((GIFunctionInfo *) info,
 			      in_args,
@@ -549,8 +614,23 @@ seed_gobject_method_invoked (JSContextRef ctx,
 
       type_info = g_callable_info_get_return_type ((GICallableInfo *) info);
       tag = g_type_info_get_tag (type_info);
-      if (tag == GI_TYPE_TAG_VOID)
-	retval_ref = JSValueMakeUndefined (ctx);
+
+      // might need to add g_type_info_is_pointer (type_info) check here..
+      
+      if (tag == GI_TYPE_TAG_VOID) 
+        {
+          // if we have no out args - returns undefined
+          // otherwise we return an object, and put the return values into that
+          // along with supporting the old object.value way
+          if (n_out_args < 1) 
+              retval_ref = JSValueMakeUndefined (ctx);
+          else 
+            {
+            
+             retval_ref = JSObjectMake (ctx, NULL, NULL);      
+             use_return_as_out = 1;
+            }
+        }
       else
 	{
 	  GIBaseInfo *interface;
@@ -609,7 +689,7 @@ seed_gobject_method_invoked (JSContextRef ctx,
       dir = g_arg_info_get_direction (arg_info);
       type_info = g_arg_info_get_type (arg_info);
 
-      if (dir == GI_DIRECTION_IN)
+      if (dir == GI_DIRECTION_IN || dir == GI_DIRECTION_INOUT) 
 	{
 	  seed_gi_release_in_arg (g_arg_info_get_ownership_transfer
 				  (arg_info), type_info,
@@ -621,15 +701,43 @@ seed_gobject_method_invoked (JSContextRef ctx,
 	  g_base_info_unref ((GIBaseInfo *) arg_info);
 	  continue;
 	}
+      
+      // we are now only dealing with out arguments.
       jsout_val = seed_gi_argument_make_js (ctx, &out_values[out_args_pos],
 					    type_info, exception);
       out_args_pos++;
-      if (!JSValueIsNull (ctx, arguments[i]) &&
-	  JSValueIsObject (ctx, arguments[i]))
-	{
-	  seed_object_set_property (ctx, (JSObjectRef) arguments[i],
-				    "value", jsout_val);
-	}
+
+
+
+      // old ? depreciated ? way to handle out args -> set 'value' on object that was send through.
+       
+      if ( (i < argumentCount) && 
+          !JSValueIsNull (ctx, arguments[i]) &&
+          JSValueIsObject (ctx, arguments[i]))
+        {
+          seed_object_set_property (ctx, (JSObjectRef) arguments[i],
+                                   "value", jsout_val);
+        }
+
+
+      // if we add it to the return argument and/or the first out arguement
+
+      if (use_return_as_out) 
+        {
+           seed_object_set_property (ctx, (JSObjectRef) retval_ref,
+                 g_base_info_get_name((GIBaseInfo*) arg_info) , jsout_val);
+        }
+       
+
+      if ( (first_out > -1) &&  
+          !JSValueIsNull (ctx, arguments[first_out]) &&
+          JSValueIsObject (ctx, arguments[first_out]) )
+       
+        {
+          seed_object_set_property (ctx, (JSObjectRef) arguments[first_out],
+                 g_base_info_get_name((GIBaseInfo*) arg_info) , jsout_val);
+        }
+
 
       g_base_info_unref ((GIBaseInfo *) arg_info);
       g_base_info_unref ((GIBaseInfo *) type_info);
@@ -1354,14 +1462,16 @@ seed_engine_destroy (SeedEngine *eng)
 }
 
 /**
- * seed_init_with_context_group:
+ * seed_init_with_context_and_group:
  * @argc: A reference to the number of arguments remaining to parse.
  * @argv: A reference to an array of string arguments remaining to parse.
+ * @context A reference to an existing JavascriptCore context
  * @group: A #SeedContextGroup within which to create the initial context.
  *
- * Initializes a new #SeedEngine. This involves initializing GLib, creating
- * an initial context (in #group) with all of the default globals, and
- * initializing various internal parts of Seed.
+ * Initializes a new #SeedEngine using an existing JavascriptCore context. 
+ * This involves initializing GLib, adding @instance to @group, adding the 
+ * default globals to the provided context, and initializing various internal
+ * parts of Seed.
  *
  * This function should only be called once within a single Seed application.
  *
@@ -1369,8 +1479,8 @@ seed_engine_destroy (SeedEngine *eng)
  *
  */
 SeedEngine *
-seed_init_with_context_group (gint * argc,
-			      gchar *** argv, JSContextGroupRef group)
+seed_init_with_context_and_group (gint * argc,
+			      gchar *** argv, JSGlobalContextRef context, JSContextGroupRef group)
 {
 
   g_type_init ();
@@ -1396,7 +1506,7 @@ seed_init_with_context_group (gint * argc,
 
   context_group = group;
 
-  eng->context = JSGlobalContextCreateInGroup (context_group, NULL);
+  eng->context = context;
   eng->global = JSContextGetGlobalObject (eng->context);
   eng->group = context_group;
   eng->search_path = NULL;
@@ -1453,6 +1563,30 @@ seed_init_with_context_group (gint * argc,
   return eng;
 }
 
+
+
+/**
+ * seed_init_with_context_group:
+ * @argc: A reference to the number of arguments remaining to parse.
+ * @argv: A reference to an array of string arguments remaining to parse.
+ * @group: A #SeedContextGroup within which to create the initial context.
+ *
+ * Initializes a new #SeedEngine. This involves initializing GLib, creating
+ * an initial context (in #group) with all of the default globals, and
+ * initializing various internal parts of Seed.
+ *
+ * This function should only be called once within a single Seed application.
+ *
+ * Return value: The newly created and initialized #SeedEngine.
+ *
+ */
+SeedEngine *
+seed_init_with_context_group (gint * argc,
+			      gchar *** argv, JSContextGroupRef group)
+{
+  return seed_init_with_context_and_group (argc, argv, JSGlobalContextCreateInGroup (group, NULL), group);
+}
+
 /**
  * seed_init:
  * @argc: A reference to the number of arguments remaining to parse.
@@ -1476,3 +1610,28 @@ seed_init (gint * argc, gchar *** argv)
   return seed_init_with_context_group (argc, argv, context_group);
 }
 
+/**
+ * seed_init_with_context:
+ * @argc: A reference to the number of arguments remaining to parse.
+ * @argv: A reference to an array of string arguments remaining to parse.
+ * @context A reference to an existing JavascriptCore context
+
+ * Initializes a new #SeedEngine using an existing JavascriptCore context. 
+ * This involves initializing GLib, adding the default globals to the provided
+ * @context and initializing various internal parts of Seed.
+ *
+ * This function should only be called once within a single Seed application.
+ *
+ * Return value: The newly created and initialized #SeedEngine.
+ *
+ */
+SeedEngine *
+seed_init_with_context (gint * argc, gchar *** argv, JSGlobalContextRef context)
+{
+  context_group = JSContextGroupCreate ();
+  pthread_key_create(&seed_next_gobject_wrapper_key, NULL);
+
+  return seed_init_with_context_and_group (argc, argv, context, context_group);
+}
+
+
